function [W,W_all] = gha(P,epochs,m,beta,W)

% function [W,W_all] = gha(P,epochs,m,beta,W)
%
%  this routine runs the GHA 
%
%   P      = pattern vectors (in rows)
%   epochs = number of epochs to train with
%   m      = number of components
%   beta   = step size try (.0001)
%   W      = optional initial weight vector
%   W_all  = stored weights in each iteration (in rows)

% Hugh Pasika 1997

% get dimensions of pattern matrix (#rows #columns)
[rP cP] = size(P);

% initialize weight matrix (unit weights in rows)
if nargin < 7, W = rand(m,cP)*(max(max(P))); end

% initalize stoage space for evolution of weights
if nargout == 2, W_all = zeros(epochs*rP,m*cP); end
iter = 1;

% run GHA
for epoch = 1:epochs,
	cc = clock;
	ind = rand(1,rP); [y inds] = sort(ind); P = P(inds,:);

	for j = 1:rP,
		x = P(j,:);
		Wd = zeros(size(W));
		y(1) = dot(W(1,:),x);
		Wd(1,:) = beta*y(1)*(x-y(1)*W(1,:));
		for h = 2:m
			y(h) = dot(W(h,:),x);
			temp = 0; 
			for k = 1:h
				temp = temp+W(k,:)*y(k); 
			end
		Wd(h,:) = beta*y(h)*(x-temp);
      end
      W = W + Wd;
	  % add current weigths to W_all
	  W_all(iter,:) = reshape(W,1,m*cP);
	  iter = iter + 1;
   end
   fprintf(1,'.');
end

W = W';
